#include <Ultrasonic.h>
#include <Servo.h>
#include <LiquidCrystal.h>

// IN from LAPSE
int M0E_PIN = 22;
int M0D_PIN = 24;
int M1E_PIN = 26;
int M1D_PIN = 28;
int SHOT_PIN = 30;
//OUT to LAPSE
int TRAVA_PIN = 34;
int ALVO_PIN = 36;

// CANHAO
int CANHAO_TRAVA_PIN = 52;

// Servos
int SERVO_HORIZ_PIN = 45;
int SERVO_VERT_PIN = 43;
int SERVO_TRIG_PIN = 41;

float SPEED_MAX = 2.0;
float SPEED_MIN = 1.0;

Servo ServoHoriz;
Servo ServoVert;
Servo ServoTrig;

int trigOff = 1900;
int trigOn = 1200;

float horizPos = 1500;
float horizMax = 2000;
float horizMin = 1000;
float horizSpeed = SPEED_MIN;

float vertPos = 1500;
float vertMax = 1600;
float vertMin = 1350;
float vertSpeed = SPEED_MIN;

//IN
int m0e = 0;
int m0d = 0;
int m1e = 0;
int m1d = 0;
int disparo = 0; // servo de disparo acionado 
// OUT
int trava = 0; // trava de seguranca
int atingido = 0; // alvo foi atingido

//SONAR
// 21 trig // verde
// 20 echo // amarelo
Ultrasonic sonar(21,20);
float distancia = 0.0;
#define MINDIST 40.0

#include "display.h"
void setup() 
{
  setupDisplay();
  ServoHoriz.attach(SERVO_HORIZ_PIN);
  ServoVert.attach(SERVO_VERT_PIN);
  ServoTrig.attach(SERVO_TRIG_PIN);
  
  ServoHoriz.writeMicroseconds(horizPos);
  ServoVert.writeMicroseconds(vertPos);
  ServoTrig.writeMicroseconds(trigOff);
  
  pinMode(TRAVA_PIN, OUTPUT);
  pinMode(ALVO_PIN, OUTPUT);
  digitalWrite(TRAVA_PIN, trava);
  digitalWrite(ALVO_PIN, atingido);

  populaDisplay();
}

int lerTecladoAnalogico()
{
  unsigned int val = analogRead(0);
  if(val < 66) return 1; // Right
  if(val < 220) return 2; // Up
  if(val < 392) return 3; // Down
  if(val < 601) return 4; // Left
  if(val < 873) return 5; // Select
  return 0;
}

void disparoManual(int bt)
{
  static int clicSel = 0;
  if( (bt == 5) && !clicSel)
  {
    clicSel = 1;
    disparo = !disparo;
    disparo ? ServoTrig.writeMicroseconds(trigOn) : ServoTrig.writeMicroseconds(trigOff);
    populaDisplay();
  }
  if(!bt) clicSel = 0;
}

void simulaSonar(int bt)
{
  static int clicSel = 0;
  if( (bt == 5) && !clicSel)
  {
    clicSel = 1;
    atingido = !atingido;
    populaDisplay();
  }
  if(!bt) clicSel = 0;
}

void processaSonar()
{
    distancia = sonar.Ranging(CM);
    (distancia > MINDIST) ? atingido = 1 : atingido = 0;
}

void processaEntradas()
{
  // LAPSE
  m0e = digitalRead(M0E_PIN);
  m0d = digitalRead(M0D_PIN);
  m1e = digitalRead(M1E_PIN);
  m1d = digitalRead(M1D_PIN);
  disparo = digitalRead(SHOT_PIN);
  
  // Canhao
  trava = digitalRead(CANHAO_TRAVA_PIN);
  //TODO: SONAR
  
  populaDisplay();
}

void processaServos()
{
  // Disparar
  disparo ? ServoTrig.writeMicroseconds(trigOn) : ServoTrig.writeMicroseconds(trigOff);
  
  // Vertical
  float delta = 0.0;
  if(m0e && !m0d) delta = SPEED_MIN;
  if(!m0e && m0d) delta = -SPEED_MIN;
  vertPos += delta;
  if(vertPos > vertMax) vertPos = vertMax;
  if(vertPos < vertMin) vertPos = vertMin;
  ServoVert.writeMicroseconds(vertPos);
  
  // Horizontal
  delta = 0.0;
  if(m1e && !m1d) delta = SPEED_MIN;
  if(!m1e && m1d) delta = -SPEED_MIN;
  horizPos += delta;
  if(horizPos > horizMax) horizPos = horizMax;
  if(horizPos < horizMin) horizPos = horizMin;
  ServoHoriz.writeMicroseconds(horizPos);
}

void processaSaidas()
{
   digitalWrite(TRAVA_PIN, trava);
   digitalWrite(ALVO_PIN, atingido);
   
   populaDisplay();
}

void loop() 
{
  int bt = lerTecladoAnalogico();
  
  processaEntradas();
  //simulaSonar(bt);
  processaSonar();
  
  processaSaidas();
  
  processaServos();
  
  atualizaDisplay();
}
